from app.utils import background_prepare_corpus_file
from flask import (abort, current_app, flash, redirect, request,
                   render_template, url_for, send_from_directory)
from flask_login import current_user, login_required
from . import corpora
from .forms import (AddCorpusFileForm, AddCorpusForm, EditCorpusFileForm,
                    QueryDownloadForm, QueryForm)
from .. import db
from ..models import Corpus, CorpusFile
from werkzeug.utils import secure_filename
import os
import threading
import logging


@corpora.route('/add', methods=['GET', 'POST'])
@login_required
def add_corpus():
    add_corpus_form = AddCorpusForm()
    if add_corpus_form.validate_on_submit():
        corpus = Corpus(creator=current_user,
                        description=add_corpus_form.description.data,
                        status='unprepared', title=add_corpus_form.title.data)
        db.session.add(corpus)
        db.session.commit()
        dir = os.path.join(current_app.config['OPAQUE_STORAGE_DIRECTORY'],
                           str(corpus.user_id), 'corpora', str(corpus.id))
        try:
            os.makedirs(dir)
        except OSError:
            flash('OSError!')
            db.session.remove(corpus)
            db.session.commit()
        flash('Corpus added!')
        return redirect(url_for('corpora.corpus', corpus_id=corpus.id))
    return render_template('corpora/add_corpus.html.j2',
                           add_corpus_form=add_corpus_form,
                           title='Add corpus')


@corpora.route('/<int:corpus_id>')
@login_required
def corpus(corpus_id):
    corpus = Corpus.query.get_or_404(corpus_id)
    if not (corpus.creator == current_user or current_user.is_administrator()):
        abort(403)
    return render_template('corpora/corpus.html.j2', corpus=corpus,
                           title='Corpus')


@corpora.route('/<int:corpus_id>/analysis', methods=['GET', 'POST'])
@login_required
def corpus_analysis(corpus_id):
    logger = logging.getLogger(__name__)
    corpus = Corpus.query.get_or_404(corpus_id)
    query = request.args.get('query')
    logger.warning('Query first: {}'.format(query))
    hits_per_page = request.args.get('hits_per_page', 30)
    context = request.args.get('context', 10)
    dl_form = QueryDownloadForm()
    form = QueryForm(hits_per_page=hits_per_page, context=context, query=query)
    if form.validate_on_submit():
        logger = logging.getLogger(__name__)
        logger.warning('Data has been sent!')
        logger.warning('Data labels: {data}'.format(data=[data for data in form.data]))
        logger.warning('Query Second: {q}'.format(q=form.query.data))
        logger.warning('Hits: {hits}'.format(hits=form.hits_per_page.data))
        logger.warning('Context: {context}'.format(context=form.context.data))
        flash('Query has been sent!')
        query = form.query.data
        hits_per_page = form.hits_per_page.data
        context = form.context.data
        logger.warning('Query Thrid: {sq}'.format(sq=query))
        return redirect(url_for('corpora.corpus_analysis', corpus_id=corpus_id,
                                query=query, hits_per_page=hits_per_page,
                                context=context))
    return render_template('corpora/corpus_analysis.html.j2', corpus=corpus,
                           form=form, dl_form=dl_form,
                           title='Corpus: {}'.format(corpus.title))


@corpora.route('/<int:corpus_id>/prepare')
@login_required
def prepare_corpus(corpus_id):
    corpus = Corpus.query.get_or_404(corpus_id)
    if not (corpus.creator == current_user or current_user.is_administrator()):
        abort(403)
    if len(corpus.files.all()) > 0:
        corpus.status = 'preparable'
        db.session.commit()
        flash('Corpus marked for preparation!')
    else:
        flash('Can not prepare corpus, please add corpus file(s).')
    return redirect(url_for('corpora.corpus', corpus_id=corpus_id))


@corpora.route('/<int:corpus_id>/delete')
@login_required
def delete_corpus(corpus_id):
    corpus = Corpus.query.get_or_404(corpus_id)
    if not (corpus.creator == current_user or current_user.is_administrator()):
        abort(403)
    delete_thread = threading.Thread(corpus.delete())
    delete_thread.start()
    flash('Corpus deleted!')
    return redirect(url_for('main.dashboard'))


@corpora.route('/<int:corpus_id>/files/add', methods=['GET', 'POST'])
@login_required
def add_corpus_file(corpus_id):
    corpus = Corpus.query.get_or_404(corpus_id)
    if not (corpus.creator == current_user or current_user.is_administrator()):
        abort(403)
    add_corpus_file_form = AddCorpusFileForm()
    if add_corpus_file_form.validate_on_submit():
        file = add_corpus_file_form.file.data
        filename = secure_filename(file.filename)
        for corpus_file in corpus.files:
            if filename == corpus_file.filename:
                flash('File already registered to this corpus.')
                return redirect(url_for('corpora.corpus', corpus_id=corpus_id))
        # Save the file
        dir = os.path.join(str(corpus.user_id), 'corpora', str(corpus.id))
        file.save(os.path.join(current_app.config['OPAQUE_STORAGE_DIRECTORY'],
                               dir, filename))

        corpus_file = CorpusFile(author=add_corpus_file_form.author.data,
                                 corpus=corpus, dir=dir, filename=filename,
                                 publishing_year=add_corpus_file_form.publishing_year.data,
                                 title=add_corpus_file_form.title.data)
        db.session.add(corpus_file)
        db.session.commit()
        background_thread = threading.Thread(
            args=(current_app._get_current_object(), corpus_file.id),
            target=background_prepare_corpus_file
        )
        background_thread.start()
        flash('Corpus file added!')
        return redirect(url_for('corpora.corpus', corpus_id=corpus_id))
    return render_template('corpora/add_corpus_file.html.j2',
                           add_corpus_file_form=add_corpus_file_form,
                           corpus=corpus, title='Add corpus file')


@corpora.route('/<int:corpus_id>/files/<int:corpus_file_id>/delete')
@login_required
def delete_corpus_file(corpus_id, corpus_file_id):
    corpus_file = CorpusFile.query.get_or_404(corpus_file_id)
    if not corpus_file.corpus_id == corpus_id:
        abort(404)
    if not (corpus_file.corpus.creator == current_user
            or current_user.is_administrator()):
        abort(403)
    delete_thread = threading.Thread(corpus_file.delete())
    delete_thread.start()
    flash('Corpus file deleted!')
    return redirect(url_for('corpora.corpus', corpus_id=corpus_id))


@corpora.route('/<int:corpus_id>/files/<int:corpus_file_id>/download')
@login_required
def download_corpus_file(corpus_id, corpus_file_id):
    corpus_file = CorpusFile.query.get_or_404(corpus_file_id)
    if not corpus_file.corpus_id == corpus_id:
        abort(404)
    if not (corpus_file.corpus.creator == current_user
            or current_user.is_administrator()):
        abort(403)
    dir = os.path.join(current_app.config['OPAQUE_STORAGE_DIRECTORY'],
                       corpus_file.dir)
    return send_from_directory(as_attachment=True, directory=dir,
                               filename=corpus_file.filename)


@corpora.route('/<int:corpus_id>/files/<int:corpus_file_id>/edit', methods=['GET', 'POST'])
@login_required
def edit_corpus_file(corpus_id, corpus_file_id):
    corpus_file = CorpusFile.query.get_or_404(corpus_file_id)
    if not corpus_file.corpus_id == corpus_id:
        abort(404)
    if not (corpus_file.corpus.creator == current_user
            or current_user.is_administrator()):
        abort(403)
    edit_corpus_file_form = EditCorpusFileForm()
    if edit_corpus_file_form.validate_on_submit():
        corpus_file.author = edit_corpus_file_form.author.data
        corpus_file.publishing_year = edit_corpus_file_form.publishing_year.data
        corpus_file.title = edit_corpus_file_form.title.data
        corpus_file.insert_metadata()
        db.session.commit()
        flash('Corpus file edited!')
        return redirect(url_for('corpora.corpus', corpus_id=corpus_id))
    return render_template('corpora/edit_corpus_file.html.j2',
                           edit_corpus_file_form=edit_corpus_file_form,
                           corpus_file=corpus_file, title='Edit corpus file')
